#define __PROG_TYPES_COMPAT__

#define F_CPU			16000000

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

#include "ST7735R_TFT.h"
#include "spi.h"
#include "math.h"
#include "mpu6050.h"
#include "i2cmaster.h"


#define BUTTON_REG		DDRA
#define BUTTON_PORT		PORTA
#define BUTTON_PIN		PINA


#define LEFT			PA3
#define RIGHT			PA0
#define UP				PA1
#define DOWN			PA4
#define FLAG			PA2

#define SELECT			PA5

#define OFFSET			25
#define NUM_NOTE		40

#define MORSE_BOMB		20
#define MORSE_LUNG		150
#define MORSE_SCURT		50
#define MORSE_PAUZA		50
#define MORSE_EXTREME   700

int level1_nr_bombs = 10;

typedef struct Cell {
	int number;			//numarul de bombe adiacente
	int colour;			//culoarea fundalului celulei (0 - albastra, 1-rosie)
	int x;				//coordonata x corespunzatoare pe ecran
	int y;				//coordonata y corespunzatoare pe ecran
	int hasBomb;		//existenta bombei in celula curenta
}C;

int frecventa_note[NUM_NOTE] = {
	659.25511, 493.8833, 523.25113, 587.32954, 523.25113, 493.8833, 440.0, 440.0,
	523.25113, 659.25511, 587.32954, 523.25113, 493.8833, 523.25113, 587.32954,
	659.25511, 523.25113, 440.0, 440.0, 440.0, 493.8833, 523.25113, 587.32954,
	698.45646, 880.0, 783.99087, 698.45646, 659.25511, 523.25113, 659.25511,
	587.32954, 523.25113, 493.8833, 493.8833, 523.25113, 587.32954, 659.25511,
	523.25113, 440.0, 440.0
};

int frecventa_nota_bomb[NUM_NOTE] = {
	33.4, 38.2, 124, 24,18.7,80.4,14.8
};

int noteDurations[NUM_NOTE] = {
	406.250, 203.125, 203.125, 406.250, 203.125, 203.125, 406.250, 203.125,
	203.125, 406.250, 203.125, 203.125, 609.375, 203.125, 406.250, 406.250,
	406.250, 406.250, 203.125, 203.125, 203.125, 203.125, 609.375, 203.125,
	406.250, 203.125, 203.125, 609.375, 203.125, 406.250, 203.125, 203.125,
	406.250, 203.125, 203.125, 406.250, 406.250, 406.250, 406.250, 406.250
};


int frecventa_nota[8] = {
	262, 294, 330, 349, 392, 440, 494, 523
};


/* Genereaza o nota folosind buzzer-ul. */
void speaker_reda(int nota) {
	int perioada_nota = 1000000 / frecventa_nota[nota];
	int i;

	
	PORTD |= (1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);

	PORTD &= ~(1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);
}

void speaker_reda2(int nota) {
	int perioada_nota = 1000000l / frecventa_note[nota];
	int i;

	/*
	 * Nu putem folosi variable ca argument pentru functia
	 * _delay_us(), asa ca folosim acest "workaround".
	 */
	PORTD |= (1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);

	PORTD &= ~(1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);
}


void play_song(void) {


	int nota_curenta = -1;

	/* Setez pinul PD4 ca iesire. */
	DDRD |= (1 << PD4);

	/* Opresc speaker-ul. */
	PORTD &= ~(1 << PD4);

	while (1) {

		if (nota_curenta < NUM_NOTE - 1)
		++nota_curenta;
		_delay_ms(10);


		if (nota_curenta != -1) {

			/* Redam nota. */
			for(int i = 0;i<noteDurations[nota_curenta]/3;i++)
			speaker_reda2(nota_curenta);
			} 
		if(nota_curenta == NUM_NOTE-1){
			break;
		}
	}
}

static void show_start()
{

	ST7735R_DrawText(20, 80, "Mihaescu Ileana", 255, 0, 0, 220, 220, 220);
	ST7735R_DrawText(40, 90, "336CB", 255, 0, 0, 220, 220, 220);
}


static void show_lost()
{
	char scor[20];
	ST7735R_DrawText(40, 10, "YOU LOST!", 255, 255, 0,255, 0, 0);	
}


static void show_start2()
{
	ST7735R_DrawText(30, 4, "Minesweeper", 0, 0, 255, 0, 0, 0);
	play_song();
}

/* Genereaza sunetul pentru tipul de ton folosit */
void speaker_morse(int tip_morse) {
	int i;

	/* Alternam starea pinului PD4 la fiecare milisecunda. */
	for (i = 0; i < tip_morse; ++i) {
		PIND = (1 << PD4);

		_delay_ms(1);
	}
}


int main(void)
{		
	// Setare butoane
	DDRA &= ~((1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5));
	PORTA |= ((1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5));
	
	
	// Setare speaker
	DDRD |= (1 << PD4);
	PORTD &= ~(1 << PD4);

	// Initialize primara LCD
	SPI_init();
	ST7735R_Begin();

	// Setare ecran initial
	ST7735R_FillRect(0, 0, ST7735R_WIDTH, ST7735R_HEIGHT, 220, 220, 220);
	
	// Afisare mesaj de inceput 
	show_start();
	
	// Delay pentru afisarea mesajului de inceput
	_delay_ms(2000);

first:
	ST7735R_FillRect(0, 0, ST7735R_WIDTH, ST7735R_HEIGHT, 0, 0, 0);
	
	int start_shown = 0, i, empty_line_i = 0, j, empty_line_j = 0;
	int cursor_x = ST7735R_WIDTH/16;
	int cursor_y = ST7735R_HEIGHT/16;
	int old_cursor_x, old_cursor_y;
	int part_x = ST7735R_WIDTH/8;
	int part_y = ST7735R_HEIGHT/8;
	int radius = 3;
	int hasStarted = 0;
	struct Cell minesweeper[6][7];
	int index_i = 0;
	int index_j = 0;
	int score = 0;
	int end = 0;
	int flag = 0;
	int revealed_piece = 0;
	int revealed_number = 0;
	int count = 0;
	int sound = 0;
	old_cursor_x = cursor_x;
	old_cursor_y = cursor_y;
	int real_score = -1;

	while (end == 0) {
	
		// Afisare meniu
		if (start_shown == 0) {
			show_start2();
			start_shown = 1;
		}


		if((PINA & (1 << SELECT)) == 0 && hasStarted == 0) {
			//generate bombs and points random
			for (i = 0; i < 6; i++) {
				for (j = 0; j < 7; j++) {
					minesweeper[i][j].hasBomb = 0;
					minesweeper[i][j].number = -1;
					minesweeper[i][j].x = ST7735R_WIDTH/8 * j + empty_line_j;
					minesweeper[i][j].y = ST7735R_HEIGHT/8 * i + empty_line_i;
					minesweeper[i][j].colour = 0;

					ST7735R_FillRect(minesweeper[i][j].x,
					ST7735R_HEIGHT/8 * i + empty_line_i,
					ST7735R_WIDTH/8* (j+1) +  empty_line_j, 
					ST7735R_HEIGHT/8 * (i+1) + empty_line_i, 250, 0, 0);	
					empty_line_j += 2;
				}

				empty_line_j = 0;
				empty_line_i += 2;
				
			}
			hasStarted = 1;
			ST7735R_FilledCircle(cursor_x, cursor_y, radius, 0, 0, 0);

			//generate bombs
			for (i = 0; i < level1_nr_bombs; i++) {
				int position_x = rand() % 6;
				int position_y = rand() % 6;
				while (minesweeper[position_x][position_y].hasBomb == 1) {
					position_x = rand() % 6;
					position_y = rand() % 6;
				}
				minesweeper[position_x][position_y].hasBomb = 1;
			}

			//generate points
			for (i = 0; i < 25; i++) {
				int position_x = rand() % 6;
				int position_y = rand() % 6;
				int number = rand() % 4;

				while (number == 0) {
					number = rand() % 10;
				}
				if (minesweeper[position_x][position_y].hasBomb == 0) {
					minesweeper[position_x][position_y].number = number;
					real_score += number;
				}
			}

			index_i = 0;
			index_j = 0;
			_delay_ms(1000);
		}

		if((PINA & (1 << RIGHT)) == 0) {
			if (revealed_piece == 0) {
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 250, 0, 0);
			}
			else {
				if (revealed_number == 0)
					ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 0, 0);
			}
			if (flag)
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 255, 255);
			revealed_piece = 0;
			revealed_number = 0;
			flag = 0;
			cursor_x += part_x + 2;
			ST7735R_FilledCircle(cursor_x, cursor_y, radius, 0, 0, 0);
			old_cursor_x = cursor_x;
			old_cursor_y = cursor_y;
			index_i++;
			_delay_ms(500);
		}

		if((PINA & (1 << LEFT)) == 0) {
			if (revealed_piece == 0) {
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 250, 0, 0);
			}
			else {
				if (revealed_number == 0)
					ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 0, 0);
			}
			if (flag)
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 255, 255);
			revealed_piece = 0;
			revealed_number = 0;
			flag = 0;
			cursor_x -= part_x + 2;
			ST7735R_FilledCircle(cursor_x, cursor_y, radius, 0, 0, 0);
			index_i--;
			old_cursor_x = cursor_x;
			old_cursor_y = cursor_y;
			_delay_ms(500);
		} 

		if((PINA & (1 << UP)) == 0) {
			if (revealed_piece == 0) {
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 250, 0, 0);
			}
			else {
				if (revealed_number == 0)
					ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 0, 0);
			}
			if (flag)
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 255, 255);
			revealed_piece = 0;
			flag = 0;
			revealed_number = 0;
			cursor_y -= part_y + 2;
			ST7735R_FilledCircle(cursor_x, cursor_y, radius, 0, 0, 0);
			index_j--;
			old_cursor_x = cursor_x;
			old_cursor_y = cursor_y;
			_delay_ms(500);
		} 

		if((PINA & (1 << DOWN)) == 0) {
			if (revealed_piece == 0) {
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 250, 0, 0);
			}
			else {
				if (revealed_number == 0)
					ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 0, 0);
			}
			if (flag)
				ST7735R_FilledCircle(old_cursor_x, old_cursor_y, radius, 0, 255, 255);
			revealed_piece = 0;
			flag = 0;
			revealed_number = 0;
			cursor_y += part_y + 2;
			index_j++;
			old_cursor_x = cursor_x;
			old_cursor_y = cursor_y;
			ST7735R_FilledCircle(cursor_x, cursor_y, radius, 0, 0, 0);
			_delay_ms(500);
		} 

		if((PINA & (1 << FLAG)) == 0) {
			flag = 1;
			count ++;
			ST7735R_FillRect(
				minesweeper[index_j][index_i].x,
				minesweeper[index_j][index_i].y,
				ST7735R_WIDTH/8 + minesweeper[index_j][index_i].x,
				ST7735R_HEIGHT/8 + minesweeper[index_j][index_i].y,
				0, 255, 255);
			_delay_ms(500);
		} 

		if (score == real_score) {
			char snum2[50];
			itoa(score, snum2, 10);
			ST7735R_DrawText(40, 90, snum2, 255, 0, 0, 220, 220, 220);
			play_song();
			level1_nr_bombs+=2;
			goto first;

		}

		if((PINA & (1 << SELECT)) == 0 && hasStarted == 1) {

			if (minesweeper[index_j][index_i].hasBomb == 1) {
				ST7735R_FillRect(
				minesweeper[index_j][index_i].x,
				minesweeper[index_j][index_i].y,
				ST7735R_WIDTH/8 + minesweeper[index_j][index_i].x,
				ST7735R_HEIGHT/8 + minesweeper[index_j][index_i].y,
				0, 0, 0);
				ST7735R_FilledCircle(
					minesweeper[index_j][index_i].x + 8.5,
					minesweeper[index_j][index_i].y + 11,
					radius + 5, 0, 0, 250);

				empty_line_i = 0;
				count += level1_nr_bombs;
				empty_line_j = 0;
				for (i = 0; i < 6; i++) {
					for (j = 0; j < 7; j++) {
						if (minesweeper[i][j].hasBomb == 1) {
							minesweeper[i][j].x = ST7735R_WIDTH/8 * j + empty_line_j;
							minesweeper[i][j].y = ST7735R_HEIGHT/8 * i + empty_line_i;

							ST7735R_FillRect(minesweeper[i][j].x,
							ST7735R_HEIGHT/8 * i + empty_line_i,
							ST7735R_WIDTH/8* (j+1) +  empty_line_j, 
							ST7735R_HEIGHT/8 * (i+1) + empty_line_i, 
							0, 0, 0);	

							ST7735R_FilledCircle(minesweeper[i][j].x + 8.5,
								ST7735R_HEIGHT/8 * i + empty_line_i + 11,
								radius+5,
								0, 0, 255);
						}
						empty_line_j += 2;
					}

					empty_line_j = 0;
					empty_line_i += 2;
					
				}
				//reda sunet bomba;
				while (1) {
					speaker_morse(MORSE_LUNG);
					speaker_morse(MORSE_SCURT);
					speaker_morse(MORSE_BOMB);
					speaker_morse(MORSE_EXTREME);
					speaker_morse(MORSE_BOMB);


					PORTD &= ~(1 << PD4);
					_delay_ms(MORSE_PAUZA);
					sound++;
					if (sound == 6)
						break;
				}
				//afisare scor trebuie facut pe 7 Segments Display care s-a ars, asa ca afisez pe ecran
				ST7735R_FillRect(0, 0, ST7735R_WIDTH, ST7735R_HEIGHT, 0, 0, 0);
				
				char scor[20];
				ST7735R_DrawText(40, 10, "YOU LOST!", 255, 255, 0,255, 0, 0);

				char snum2[50];
				itoa(score, snum2, 10);
				ST7735R_DrawText(60, 20, snum2, 255, 0, 0, 220, 220, 220);
				_delay_ms(2000);

				//de la inceput
				goto first;
				end = 1;
			}

			if (minesweeper[index_j][index_i].number != -1 && end == 0) {
					score += minesweeper[index_j][index_i].number;

					ST7735R_FillRect(
						minesweeper[index_j][index_i].x,
						minesweeper[index_j][index_i].y,
						ST7735R_WIDTH/8 + minesweeper[index_j][index_i].x-2,
						ST7735R_HEIGHT/8 + minesweeper[index_j][index_i].y-2,
						0, 0, 0);

					char charNumber[50];

					itoa(minesweeper[index_j][index_i].number, charNumber, 10);

					ST7735R_DrawText(minesweeper[index_j][index_i].x + 8.5,
					minesweeper[index_j][index_i].y + 11, charNumber, 0, 225, 225, 0, 0, 0);
					revealed_piece = 1;
					revealed_number = 1;
					count ++;
					_delay_ms(1000);
			}

			if (minesweeper[index_j][index_i].number == -1 && end == 0	) {
				ST7735R_FillRect(
						minesweeper[index_j][index_i].x,
						minesweeper[index_j][index_i].y,
						ST7735R_WIDTH/8 + minesweeper[index_j][index_i].x-2,
						ST7735R_HEIGHT/8 + minesweeper[index_j][index_i].y-2,
						0, 0, 0);
				revealed_piece = 1;
				count++;
			}
		}
	}

	return 0;
}
